Attribute VB_Name = "modResizer"
'========================================================================
' Module:   CtrlResizer
' Author:   Carlo Somigliana (Italy)
' e-mail:   somic@libero.it
' Rel.  :   0.30
' Date  :   15 May 2006
' lang. :   Visual Basic 5.0 - 6.0
'
' Description:
'   A VB module to automatically resize controls with forms and
'   ChilForms with their MDI Forms
'   Change Maxf constant to change max AutoResizing forms
'
' Use:
'   -Add 'AutoResize' in the TAG property of the controls you want to
'   automatically resize or Set ResizeAllControls flag to TRUE to
'   automatically resize all the controls in the form.
'   -Add 'NoResize' in the TAG property of the controls you DON'T want
'   to be automatically resized
'   -Add 'NoFontResize' in the TAG property of the controls you DON'T want
'   to automatically resize the Font
'   See sample code
'
' Suggestions:
'   -Use True Type Fonts (TTF) for the control. They resize better.
'   -Tip: set the Font you like in the Form's Font Property.
'         It will be the default Font for the next controls you add
'
' License:
'   Free, at the condition to leave this Module as is or update
'   the Version History at any change and advise me back about it
'   (continuous improvement !?!)
'   Comments and suggestions will be appreciated
'
' Version History
'-----------------------------------------------------------------------------------------------
' Rel.  Date          Author            Description                     Control
'                                                                       Dipendent Code
'
' 0.00  23 Apr 2001   C. Somigliana     First Issue                         -
' 0.10  04 May 2001          "          Modified for MDI Forms              No
'                                       (req. Gawie Wolmarans)
' 0.11  13 Jul 2001          "          Forms dimensions stored             No
'                                       only if form visible
' 0.20  11 Nov 2001          "          Fixed Bug with Line control         No
'                                       (thanks to Massimo Riccardi)
' 0.21  17 Nov 2001          "          Added Form Font Resize              No
' 0.22  27 Dec 2001          "          Added
'                                       -Hidden Form while Resizing         ---
'                                       -NoFontResize parameter in          No
'                                       in .Tag to void resizing font
'                                       -DeleteFormStartDimensions          No
' 0.23  22 Jan 2002          "          -Fixed a bug Iconizing form         No
' 0.24  28 Apr 2002          "          -Added 'NoResize' and               No
'                                        'NoFontResize' options
'                                       -Added check if control can't
'                                        be resized (es. Timer)
' 0.24B 02 May 2002          "          -Added SSTab support                Yes
'                                       (req. by father Zdzislaw
'                                       Huber-Finland)
' 0.24C 01 Jul 2002   Kipp E. Howard    Fixed a bug in Sub                  No
'                                       SetNewControlsDimensions
'                                       (many thanks)
' 0.25  11 Aug 2002   C. Somigliana     -Hidden Form while Resizing         No
'                                       only if form is Not Modal
'                                       -Code Cleaned Up
'                                       -Changed Version History info
' 0.26  21 Nov 2002   U. Wilde          -Added Statusbar support            Yes
'                                       (changed lines marked with 'u.w.'
' 0.27  11 Jul 2003   C. Somigliana     -Fixed a bug of DAO Data Controls   No
'                                       in a SSTab control
'                                       (thanks to Andrei Erochok)
'                                       -Added 'suggestions'
' 0.28  01 Sep 2003   C. Somigliana     -IsChild now calling an API funct.  No
' 0.29  17 May 2004   C. Somigliana     -IsMDIForm bug corrected.           No
'                                       thanks to Steve Lillywhite
' 0.30  15 May 2006   C. Somigliana     -Fixed a bug in procedure           Yes
'                                        StoreStartControlsDimensions
'                                        causing RunTime error when opening
'                                        and closing more than MaxF times
'                                        a resizable form
'===============================================================================================
Option Explicit
'Forms' variables
Global Const MaxF = 12 'max number of AutoResizing forms
Global frmNames$(MaxF), FormsStored$
Global frmWidthStart&(MaxF), frmHeightStart&(MaxF), frmFontStart(MaxF)
'Controls' variables
Global ctrlLeftStart&(), ctrlTopStart&(), ctrlHeightStart&(), ctrlWidthStart&(), ctrlFontSize&()
Global StdFrmWidthINI&, StdFrmHeightINI& 'for the MDIForm
Const GWL_STYLE = (-16)
Const GWL_EXSTYLE = (-20)
Global ctrlItemWidthStart&() ' u.w.

Private Declare Function GetWindowLong Lib "user32" _
    Alias "GetWindowLongA" ( _
    ByVal hwnd As Long, _
    ByVal nIndex As Long) As Long

Private Declare Function IsChild Lib "user32" ( _
    ByVal hWndParent As Long, _
    ByVal hwnd As Long) As Long

'this procedure is called by the form that wants to be AutoResizable
'from the Resize Event
Sub ResizeControls(frm As Form, ResizeAllControls&)
Dim F&, AutoRedr As Boolean, varModal As Long

'at the end of Form_Load event, the form is visible and resize if called the last time
If frm.Visible = False Then Exit Sub

If InStr(1, FormsStored, " " + frm.Name + " ", vbTextCompare) = 0 Then
  Call StoreStartControlsDimensions(frm)
Else
  'if iconized, doesn't resize anything
  If frm.WindowState = vbMinimized Then Exit Sub
  
  If frm.Visible Then
    'version Hidding (to improve performances)
    'varModal = FormMode(frm.hwnd)
    'Hide the form only if Not Modal to avoid error 405 attempting to show the modal form again
    'this because we should Hide the form and then ReShow it, but the upper frm.Show Modal
    'instruction has not been completed yet, and the Hide/Show operation is not allowed.
    'If varModal = 0 Then frm.Visible = False: If Not IsMDIForm(frm) Then frm.Refresh
    Call SetNewControlsDimensions(frm, ResizeAllControls&)
    'If varModal = 0 Then frm.Show 0
  Else
    Call SetNewControlsDimensions(frm, ResizeAllControls&)
  End If
End If

End Sub
'store the starting control dimensions
'this is done only once at the first call of the ResizeControls subroutine
'the new dimensions are calculated always relatively to the original ones
'
'we could also resize the controls each time relatively to the previous dimensions,
'(avoiding to store the initial dimensions), but it wouldn't work properly with the
'NOT TrueType fonts, because they could alter the resize of the control
'
Sub StoreStartControlsDimensions(frm As Form, Optional ZoomF! = 1)
Dim k&, i&, F&, c&, ctrl As Control, FormDeleted As Boolean
Dim pcount As Integer ' u.w.
Dim pidx As Integer   ' u.w.
Static frmID&, MaxC&, MaxfrmID&

    If IsMDIForm(frm) Then
        'store MDI Form dimensions (there can be only one MDI form in the App)
        FormsStored = FormsStored + " " + frm.Name + " "
        StdFrmWidthINI = frm.Width
        StdFrmHeightINI = frm.Height
    Else
    
    FormDeleted = False
    'cerca il form nell'array dei form caricati
    For i = 1 To MaxfrmID
        'if form name doesn't exist in the FormsStored but exists in the array, then it has been deleted
        If frm.Name = frmNames(i) Then FormDeleted = True: Exit For
    Next i
    
    'set or increase form counter
    If FormDeleted Then ' se il form e' stato cancellato
        frmID = i 'lo riutilizza
    Else
        'altrimenti aggiunge una posizione nell'array dei form
        frmID = MaxfrmID + 1: MaxfrmID = frmID
    End If
    'Debug.Print frmID, MaxfrmID
    
    frmNames(frmID) = frm.Name
    If frmID > MaxF Then Exit Sub
    
    'Debug.Print FormsStored, frmID, frmNames(frmID), frm.Name
    
    'eventually redim arrays
    c = frm.Controls.Count
    If c > MaxC Then
        MaxC = c 'eventually redim arrays storing control's dimensions
        ReDim Preserve ctrlFontSize(MaxF, MaxC), ctrlLeftStart(MaxF, MaxC), _
            ctrlTopStart(MaxF, MaxC), ctrlHeightStart(MaxF, MaxC), ctrlWidthStart(MaxF, MaxC)
    End If
    
    'store form's dimensions
    FormsStored = Trim(FormsStored) + " " + frmNames(frmID) + " "
    frmWidthStart(frmID) = frm.Width
    frmHeightStart(frmID) = frm.Height
    frmFontStart(frmID) = frm.Font.Size
    'Debug.Print "Store", FormsStored
      
    On Error Resume Next 'some properties may not exist for some controls
    'store controls' dimensions
    For k = 0 To c - 1
        Set ctrl = frm.Controls(k)
        
        'check if control can be resized
        'es. Timer, CrystalReport, Menu Then GoTo skipResize
        If Not ResizableControl(ctrl) Then GoTo skipStore
        With ctrl
          
            If TypeOf ctrl Is Line Then
                ctrlLeftStart&(frmID, k) = .X1
                ctrlTopStart&(frmID, k) = .Y1
                ctrlWidthStart&(frmID, k) = .X2 - .X1
                ctrlHeightStart&(frmID, k) = .Y2 - .Y1
            Else
                ctrlLeftStart&(frmID, k) = .Left
            
                'SSTab change Left property to Hide the control
                '///
                '/// activate the following 3 lines only if you have a SSTab container
                '///
                'If TypeOf ctrl.Container Is SSTab Then
                    'condition splitted because if the first fails (no valide property),
                    'the next line (second IF) is evaluated
                    'on the same line, the next line would be the assignement instruction, not to be execute
                '    If ctrlLeftStart&(frmID, k) < 0 Then
                '        ctrlLeftStart&(frmID, k) = ctrlLeftStart&(frmID, k) + 75000
                '    End If
                'End If
                
                ctrlTopStart&(frmID, k) = .Top
                ctrlHeightStart&(frmID, k) = .Height
                ctrlWidthStart&(frmID, k) = .Width
            
                '///
                '/// activate the following 8 lines only if you have a StatusBar control
                '///
        ' u.w.
        '        If TypeOf ctrl Is StatusBar Then
        '          pcount = ctrl.Panels.Count
        '          ReDim Preserve ctrlItemWidthStart(MaxF, MaxC, pcount)
        '          For pidx = 0 To pcount - 1
        '            ctrlItemWidthStart(frmID, k, pidx) = ctrl.Panels.Item(pidx).Width
        '          Next
        '        End If
        ' u.w. (end)
        '        If TypeOf ctrl Is Toolbar Then
        '          pcount = ctrl.Buttons.Count
        '          If pcount > UBound(ctrlItemWidthStart, 2) And pcount > 0 Then
        '            ReDim Preserve ctrlItemWidthStart(MaxF, MaxC, pcount)
        '          End If
        '          For pidx = 0 To pcount - 1
        '            ctrlItemWidthStart(frmID, k, pidx) = ctrl.Buttons.Index(pidx).Width
        '          Next
        '        End If
          End If
          ctrlFontSize&(frmID, k) = .Font.Size
        End With
skipStore:
    Next k
    On Error GoTo 0
    End If
  
End Sub

'set new (or original) controls' dimensions
Sub SetNewControlsDimensions(frm As Form, AllControls&, Optional OriginalSize)
Dim k&, DHp#, DWp#, Dc#, c&, F&, ctrl As Control, StdFrm As Form
'Dim ActCtrl As Control, H2&, W2&, L2&, T2&
Dim pcount As Integer ' u.w.
Dim pidx As Integer   ' u.w.


If IsMissing(OriginalSize) Then OriginalSize = False

If IsMDIForm(frm) Then
  'look for the index of the MDI Form (Parent)
  'to calc form's dimensions variation
  For F = 0 To MaxF
    Set StdFrm = Forms(F)
    If IsMDIForm(StdFrm) Then
      If OriginalSize Then
        If StdFrm.WindowState <> 0 Then StdFrm.WindowState = 0
        StdFrm.Width = StdFrmWidthINI
        StdFrm.Height = StdFrmHeightINI
      End If
      'calc form's dimensions variation
      DWp = StdFrm.Width / StdFrmWidthINI
      DHp = StdFrm.Height / StdFrmHeightINI
      Exit For
    End If
  Next F
  
  'resize all the forms in the MDI Form
  For F = 0 To Forms.Count - 1
    Set StdFrm = Forms(F)
    If IsChild(frm.hwnd, StdFrm.hwnd) <> 0 Then
      If AllControls Or InStr(1, StdFrm.Tag, "AutoResize", vbTextCompare) > 0 _
        And InStr(1, StdFrm.Tag, "NoResize", vbTextCompare) = 0 Then
      If StdFrm.WindowState <> 0 Then Exit For
      With StdFrm
        'font first
        If InStr(1, StdFrm.Tag, "NoFontResize", vbTextCompare) = 0 And Not IsMDIForm(frm) Then
          .Font.Size = frmFontStart(F) * DHp 'FontSize follows the Height property
        End If
        'resizing must be in this order
        .Height = frmHeightStart(F) * DHp
        .Width = frmWidthStart(F) * DWp
      End With
      End If
    End If
  Next F
Else
  
  'look for the index of the actual form
  For F = 1 To MaxF
    If frmNames(F) = frm.Name Then
      If OriginalSize Then
        If frm.WindowState <> 0 Then frm.WindowState = 0
        frm.Width = frmWidthStart(F)
        frm.Height = frmHeightStart(F)
      End If
      'calc form's dimensions variation
      DWp = frm.Width / frmWidthStart(F)
      DHp = frm.Height / frmHeightStart(F)
      Dc = DWp 'Sqr(DHp * DWp)
      If InStr(1, frm.Tag, "NoFontResize", vbTextCompare) = 0 Then
        frm.Font.Size = frmFontStart(F) * Dc
          '.Font.Size = frmFontStart(f) * DHp 'FontSize follows the Width property
      End If

      Exit For
    End If
  Next F
  
  
  'count controls
  c = frm.Controls.Count
  On Error Resume Next 'some properties are ReadOnly for some controls
  'cycle trou the controls on the form
  For k = 0 To c - 1
    Set ctrl = frm.Controls(k)
    'If ctrl.Name = "Data_Familien" Then Stop
    If AllControls Or InStr(1, ctrl.Tag, "AutoResize", vbTextCompare) > 0 _
              And InStr(1, ctrl.Tag, "NoResize", vbTextCompare) = 0 Then
      'check if control can be resized
      'es. Timer, CrystalReport, Menu Then GoTo skipResize
      If Not ResizableControl(ctrl) Then GoTo skipResize
  
      With ctrl
        'font first
        Dc = DWp 'Sqr(DHp * DWp)
        If InStr(1, ctrl.Tag, "NoFontResize", vbTextCompare) = 0 Then
          .Font.Size = ctrlFontSize(F, k) * Dc
        End If
        'resizing must be in this order
        If TypeOf ctrl Is Line Then
          .X1 = ctrlLeftStart(F, k) * DWp
          .Y1 = ctrlTopStart(F, k) * DHp
          .X2 = .X1 + ctrlWidthStart(F, k) * DWp
          .Y2 = .Y1 + ctrlHeightStart(F, k) * DHp
        Else
          'ctrl.Move L2, T2, W2, H2'Move doesn't work properly
          
          .Height = ctrlHeightStart(F, k) * DHp
          .Width = ctrlWidthStart(F, k) * DWp
          'only if >=0 because if negative it is not visible.
          'some controls (es. SSTab) put this corrdinate negative to hide the control
          If .Left >= 0 Then .Left = ctrlLeftStart(F, k) * DWp
          .Top = ctrlTopStart(F, k) * DHp
        End If
        
        '///
        '/// activate the following 8 lines only if you have a StatusBar control
        '///
' u.w.
'        If TypeOf ctrl Is StatusBar Then
'          pcount = ctrl.Panels.Count
'          For pidx = 0 To pcount - 1
'            ctrl.Panels.Item(pidx).Width = ctrlItemWidthStart(f, k, pidx) * DWp
'          Next
'        End If
' u.w. (end)
'        Dim t As Toolbar
'        If TypeOf ctrl Is Toolbar Then
'            pcount = ctrl.Buttons.Count
'            For pidx = 0 To pcount - 1
'                ctrl.Buttons.Item(pidx).Width = ctrlItemWidthStart(f, k, pidx) * DWp
'                'ctrl.Buttons.Item(pidx).Width = ctrlItemHeightStart(f, k, pidx) * DHp
'            Next
'            ctrl.ButtonHeight = ctrlHeightStart(f, k) * DHp
'        End If
      
      End With
skipResize:
    End If
  Next k
  frm.Refresh
  
  On Error GoTo 0

End If

End Sub

Sub ResetControlsDimensions(frm As Form)
  
  Call SetNewControlsDimensions(frm, True, True)

End Sub

'Old code (superseeded by API function: IsChild )
Function IsChildForm&(frm As Form)
Dim c&
  
    On Error Resume Next
    'this property exist only in Standard forms
    c = frm.MDIChild
    If Err = 0 Then IsChildForm = True Else IsChildForm = False
    On Error GoTo 0

End Function


Function IsMDIForm&(frm As Form)
  
    'IsMDIForm = (IsChild(ParentHwnd, hwnd) = 0)
    Err.Clear
    On Error Resume Next
    IsMDIForm = frm.MDIChild 'esiste solo se MDIChild esiste, ossia per i Child di form MIDI
    If Err.Number > 0 Then IsMDIForm = True Else IsMDIForm = False
    On Error GoTo 0

End Function

'delete form name from the list of stored forms
Sub DeleteFormStartDimensions(FName$)
Dim pos&, st$

    pos = InStr(1, FormsStored, FName, vbTextCompare)
    If pos > 0 Then
        st = Mid$(FormsStored, 1, pos - 1) + Mid$(FormsStored, pos + Len(FName) + 1)
        st = " " & Trim(st) & " "
    End If
    
    FormsStored = " " & Trim(st) & " "
    'Debug.Print "Delete", FormsStored
End Sub

'check if control can be resized
'es. Timer, CrystalReport, Menu Then GoTo skipResize
Function ResizableControl(ctrl As Control) As Boolean
Dim test&

    On Error Resume Next
    
    Err = 0
    test = ctrl.Left  'this instruction gives error if the control is not resizable or it is a graphical object
    If Err = 0 Then
        ResizableControl = True
    Else
        Err = 0
        test = ctrl.X1 'control is a graphical object
        If Err = 0 Then
            ResizableControl = True
        Else
            ResizableControl = False
        End If
    End If
    On Error GoTo 0
End Function

' return form mode (0=Not Modal, 1=Modal)
' Note: if an application has only one visible
'       form, this function considers it as Not Modal
'       correctly, because it can be hidden without
'       stopping the execution
        
Function FormMode(hwnd As Long) As Long
    Const NotModalStyle = &H40000 'NotModalStyle
                
    FormMode = -((GetWindowLong(hwnd, GWL_EXSTYLE) And NotModalStyle) = 0)

End Function

